/*-----------------------------------------------------------------------

File  : cto_proof.h

Author: Stephan Schulz

Contents
 
  Proof objects describing the state of a proof attempt.

Changes

<1> Sat Jul  5 02:28:25 MET DST 1997
    New

-----------------------------------------------------------------------*/

#ifndef CTO_PROOF

#define CTO_PROOF


#include <che_heuristics.h>
#include <che_lpogen.h>
#include <che_to_weightgen.h>

/*---------------------------------------------------------------------*/
/*                    Data type declarations                           */
/*---------------------------------------------------------------------*/

typedef struct proofstatecell
{
   Sig_p             signature;
   TermOrdering      ordertype;
   TOWeightGenMethod to_weight_gen;
   long              to_const_weight;
   OCB_p             ordering;
   HCB_p             heuristic;
   bool              use_indexing;
   TB_p              original_terms;
   TB_p              terms;
   TB_p              tmp_terms;
   VarBank_p         freshvars;
   ClauseSet_p       axioms;
   ClauseSet_p       processed_pos_rules;
   ClauseSet_p       processed_pos_eqns;
   ClauseSet_p       processed_neg_units;
   ClauseSet_p       processed_non_units;
   ClauseSet_p       unprocessed;
   ClauseSet_p       tmp_store;
   ClauseSet_p       demods[3];
   RewriteLevel      forward_demod;
   long              processed_count;
   long              trivial_count;
   long              forward_subsumed_count;
   long              non_trivial_count;
   long              backward_subsumed_count;
   long              backward_rewritten_count;
   long              generated_count;
   long              non_trivial_generated_count;
   long              paramod_count;
   long              factor_count;
   long              resolv_count;
}ProofStateCell, *ProofState_p;



/*---------------------------------------------------------------------*/
/*                Exported Functions and Variables                     */
/*---------------------------------------------------------------------*/

#define ProofStateCellAlloc() \
   (ProofStateCell*)SizeMalloc(sizeof(ProofStateCell))
#define ProofStateCellFree(junk) \
   SizeFree(junk, sizeof(ProofStateCell))

ProofState_p ProofStateAlloc();
void         ProofStateFree(ProofState_p junk);

OCB_p        ProofStateCreateOrdering(ProofState_p state);
void         ProofStateStatisticsPrint(FILE* out, ProofState_p state);
void         ProofStatePrint(FILE* out, ProofState_p state);

#endif

/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/






