/*-----------------------------------------------------------------------

File  : che_clausesetfeatures.c

Author: Stephan Schulz

Contents
 
  Functions for calculating certain features of clause sets.

Changes

<1> Tue Sep 29 02:50:34 MET DST 1998
    New

-----------------------------------------------------------------------*/

#include "che_clausesetfeatures.h"


/*---------------------------------------------------------------------*/
/*                        Global Variables                             */
/*---------------------------------------------------------------------*/


/*---------------------------------------------------------------------*/
/*                      Forward Declarations                           */
/*---------------------------------------------------------------------*/


/*---------------------------------------------------------------------*/
/*                         Internal Functions                          */
/*---------------------------------------------------------------------*/



/*---------------------------------------------------------------------*/
/*                         Exported Functions                          */
/*---------------------------------------------------------------------*/

/*-----------------------------------------------------------------------
//
// Function: ClauseSetCountUnit()
//
//   Count the unit clauses in set.
//
// Global Variables: -
//
// Side Effects    : -
//
/----------------------------------------------------------------------*/

long ClauseSetCountUnit(ClauseSet_p set)
{
   long     res = 0;
   Clause_p handle;
   
   for(handle = set->anchor->succ; handle!=set->anchor; handle =
	  handle->succ)
   {
      if(ClauseIsUnit(handle))
      {
	 res++;
      }
   }
   return res;
}


/*-----------------------------------------------------------------------
//
// Function: ClauseSetCountHorn()
//
//   Count the unit clauses in set.
//
// Global Variables: -
//
// Side Effects    : -
//
/----------------------------------------------------------------------*/

long ClauseSetCountHorn(ClauseSet_p set)
{
   long     res = 0;
   Clause_p handle;
   
   for(handle = set->anchor->succ; handle!=set->anchor; handle =
	  handle->succ)
   {
      if(ClauseIsHorn(handle))
      {
	 res++;
      }
   }
   return res;
}


/*-----------------------------------------------------------------------
//
// Function: ClauseSetGoalsAreGround()
//
//   Return true if all goal clauses in set are ground, false
//   otherwise. 
//
// Global Variables: -
//
// Side Effects    : -
//
/----------------------------------------------------------------------*/

bool ClauseSetGoalsAreGround(ClauseSet_p set)
{
   Clause_p handle;
   
   for(handle = set->anchor->succ; handle!=set->anchor; handle =
	  handle->succ)
   {
      if(ClauseIsGoal(handle)&&!ClauseIsGround(handle))
      {
	 return false;
      }
   }
   return true;
}


/*-----------------------------------------------------------------------
//
// Function: ClauseSetAddSymbolDistribution()
//
//   Count the occurrences of function symbols in set.
//
// Global Variables: -
//
// Side Effects    : -
//
/----------------------------------------------------------------------*/

void ClauseSetAddSymbolDistribution(ClauseSet_p set, long *dist_array)
{
   Clause_p handle;
   
   for(handle = set->anchor->succ; handle!=set->anchor; handle =
	  handle->succ)
   {
      ClauseAddSymbolDistribution(handle, dist_array);
   }
}


/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/


